@@ -18,30 +18,31 @@ pub mod pipeline {
     use crate::filter;
     use crate::lang::*;
     use crate::operator;
+    use crate::operator::sort;
     use crate::printer::{agg_printer, raw_printer};
     use crate::render::{RenderConfig, Renderer, TerminalConfig};
     use crate::typecheck::{TypeCheck, TypeError};
+    use anyhow::Error;
     use crossbeam_channel::{bounded, Receiver, RecvTimeoutError, Sender};
-    use failure::Error;
-    use failure::Fail;
     use std::collections::VecDeque;
     use std::io::{BufRead, Write};
     use std::thread;
     use std::time::Duration;
+    use thiserror::Error;
 
-    #[derive(Debug, Fail)]
+    #[derive(Debug, Error)]
     pub enum CompileError {
-        #[fail(display = "Failed to parse query")]
+        #[error("Failed to parse query")]
         Parse,
 
-        #[fail(display = "Non aggregate operators can't follow aggregate operators")]
+        #[error("Non aggregate operators can't follow aggregate operators")]
         NonAggregateAfterAggregate,
 
-        #[fail(display = "Unexpected failure: {}", message)]
+        #[error("Unexpected failure: {}", message)]
         Unexpected { message: String },
     }
 
-    #[derive(Clone, PartialEq)]
+    #[derive(Clone, PartialEq, Eq)]
     pub enum OutputMode {
         Legacy,
         Logfmt,
@@ -71,15 +72,15 @@ pub mod pipeline {
             pipeline: &QueryContainer,
         ) -> Result<Box<dyn operator::AggregateOperator>, TypeError> {
             let mode = match op.direction {
-                SortMode::Ascending => operator::SortDirection::Ascending,
-                SortMode::Descending => operator::SortDirection::Descending,
+                SortMode::Ascending => sort::SortDirection::Ascending,
+                SortMode::Descending => sort::SortDirection::Descending,
             };
             let sort_cols: Vec<operator::Expr> = op
                 .sort_cols
                 .into_iter()
                 .map(|expr| expr.type_check(pipeline))
                 .collect::<Result<Vec<_>, _>>()?;
-            Ok(Box::new(operator::Sorter::new(sort_cols, mode)))
+            Ok(Box::new(sort::Sorter::new(sort_cols, mode)))
         }
 
         fn convert_multi_agg(
@@ -118,7 +119,7 @@ pub mod pipeline {
                     multi_agg
                         .aggregate_functions
                         .iter()
-                        .map(|&(ref k, _)| Expr::column(k)),
+                        .map(|(k, _)| Expr::column(k)),
                 )
                 .collect();
 
@@ -133,8 +134,7 @@ pub mod pipeline {
             output: W,
             output_mode: OutputMode,
         ) -> Result<Self, Error> {
-            let parsed = pipeline.parse().map_err(|_pos| CompileError::Parse);
-            let query = parsed?;
+            let query = pipeline.parse()?;
             let filters = convert_filter(query.search);
             let mut in_agg = false;
             let mut pre_agg: Vec<Box<dyn operator::UnaryPreAggOperator>> = Vec::new();
@@ -165,14 +165,13 @@ pub mod pipeline {
                         if let Ok(op) = Pipeline::convert_multi_agg(agg_op, pipeline) {
                             post_agg.push(op);
 
-                            let needs_sort = match op_deque.front() {
+                            let needs_sort = matches!(
+                                op_deque.front(),
                                 Some(Operator::Inline(Positioned {
                                     value: InlineOperator::Limit { .. },
                                     ..
-                                })) => true,
-                                None => true,
-                                _ => false,
-                            };
+                                })) | None
+                            );
                             if needs_sort {
                                 post_agg.push(Pipeline::convert_sort(sorter, pipeline)?);
                             }
@@ -196,7 +195,7 @@ pub mod pipeline {
             let raw_printer =
                 raw_printer(&output_mode, render_config.clone(), TerminalConfig::load())?;
             let agg_printer = agg_printer(&output_mode, render_config, TerminalConfig::load())?;
-            Result::Ok(Pipeline {
+            Ok(Pipeline {
                 filter: filters,
                 pre_aggregates: pre_agg,
                 aggregators: post_agg,
@@ -248,7 +247,7 @@ pub mod pipeline {
 
                 if renderer.should_print() {
                     let result =
-                        renderer.render(&Pipeline::run_agg_pipeline(&head, &mut rest), false);
+                        renderer.render(&Pipeline::run_agg_pipeline(&*head, &mut rest), false);
 
                     if let Err(e) = result {
                         eprintln!("error: {}", e);
@@ -256,11 +255,10 @@ pub mod pipeline {
                     }
                 }
             }
-            let result = renderer.render(&Pipeline::run_agg_pipeline(&head, &mut rest), true);
+            let result = renderer.render(&Pipeline::run_agg_pipeline(&*head, &mut rest), true);
 
             if let Err(e) = result {
                 eprintln!("error: {}", e);
-                return;
             }
         }
 
@@ -335,7 +333,7 @@ pub mod pipeline {
         }
 
         pub fn run_agg_pipeline(
-            head: &Box<dyn operator::AggregateOperator>,
+            head: &dyn operator::AggregateOperator,
             rest: &mut [Box<dyn operator::AggregateOperator>],
         ) -> Row {
             let mut row = Row::Aggregate((*head).emit());
